/*----------------------------------------------------------------------- 

File Name: VectorFactories.c

<lalVerbatim file="VectorFactoriesCV">
Revision: $Id: VectorFactories.m4,v 1.5 2003/09/29 00:33:24 dwchin Exp $
</lalVerbatim>

-------------------------------------------------------------------------*/

/* <lalLaTeX>

\subsection{Module \texttt{VectorFactories.c}}
\label{ss:VectorFactories.c}

Create/destroy $\langle\mbox{datatype}\rangle$Vector objects. 

\subsubsection*{Prototypes}
\vspace{0.1in}
\input{VectorFactoriesD}
\idx{LALZCreateVector()}
\idx{LALCCreateVector()}
\idx{LALDCreateVector()}
\idx{LALSCreateVector()}
\idx{LALI2CreateVector()}
\idx{LALI4CreateVector()}
\idx{LALI8CreateVector()}
\idx{LALU2CreateVector()}
\idx{LALU4CreateVector()}
\idx{LALU8CreateVector()}
\idx{LALCHARCreateVector()}
\idx{LALCreateVector()}
\idx{LALZDestroyVector()}
\idx{LALCDestroyVector()}
\idx{LALDDestroyVector()}
\idx{LALSDestroyVector()}
\idx{LALI2DestroyVector()}
\idx{LALI4DestroyVector()}
\idx{LALI8DestroyVector()}
\idx{LALU2DestroyVector()}
\idx{LALU4DestroyVector()}
\idx{LALU8DestroyVector()}
\idx{LALCHARDestroyVector()}
\idx{LALDestroyVector()}

\subsubsection*{Description}

The \texttt{CreateVector} family of functions create a
$\langle\mbox{datatype}\rangle$\texttt{Vector} of the appropriate
dimensions.

The \texttt{ResizeVector} family of functions changes the amount of
storage allocated by the \texttt{CreateVector} functions.

The \texttt{DestroyVector} family of functions return the storage allocated by
the \texttt{CreateVector} functions to the system.

\subsubsection*{Algorithm}

\subsubsection*{Uses}
\begin{verbatim}
LALMalloc()
LALFree()
\end{verbatim}

\subsubsection*{Notes}

\vfill{\footnotesize\input{VectorFactoriesCV}}

</lalLaTeX> */

#include "LALStdlib.h"
#include "AVFactories.h"

/* <lalVerbatim file="VectorFactoriesNRCSID"> */
NRCSID( VECTORFACTORIESC, "$Id: VectorFactories.m4,v 1.5 2003/09/29 00:33:24 dwchin Exp $" );
/* </lalVerbatim> */

















/* <lalVerbatim file="VectorFactoriesD"> */
void LALZCreateVector ( LALStatus *status, COMPLEX16Vector **vector, UINT4 length ) 
{ /* </lalVerbatim> */
  /* 
   * Initialize status structure
   */

  INITSTATUS( status, "LALZCreateVector", VECTORFACTORIESC );	
      
  /* Check sequence length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT( length > 0, status, AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH );

  /* 
   * Check return structure: If return pointer does not point to a
   *    valid pointer then report an error 
   */

  ASSERT( vector != NULL, status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR );

  ASSERT( *vector == NULL, status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR );

  /*
   * Allocate pointer
   */

  *vector = ( COMPLEX16Vector * ) LALMalloc( sizeof( COMPLEX16Vector ) );
  if ( NULL == *vector )
  {
    ABORT( status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC );
  }

  (*vector)->length = 0;	/* length 0 until storage allocated */
  (*vector)->data   = NULL;	/* NULL data until allocated */

  /* 
   * Allocate storage 
   * Test that storage is properly allocated. Can't handle with ASSERT
   * since we need to de-allocate structure pointer before an error return
   */

  (*vector)->data = ( COMPLEX16 * ) LALMalloc( length*sizeof( COMPLEX16 ) );

  if ( NULL == (*vector)->data )
  {
    /* Must free storage pointed to by *vector */
    LALFree( *vector );
    ABORT( status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC );
  }
  (*vector)->length = length;	/* Set length if storage allocated */

  /* We be done: Normal exit */

  RETURN( status );
}


















/* <lalVerbatim file="VectorFactoriesD"> */
void LALZResizeVector ( LALStatus *status, COMPLEX16Vector **vector, UINT4 length ) 
{ /* </lalVerbatim> */
  COMPLEX16 *p; /* temporary pointer */

  /* 
   * Initialize status structure
   */
  INITSTATUS( status, "LALZResizeVector", VECTORFACTORIESC );	

  ASSERT ( vector != NULL, status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR );
      
  /* Want this to behave like realloc(3), i.e.
   * *vector == NULL => create a new vector 
   * length == 0 => destroy the vector 
   * otherwise => resize given vector 
   */
  if ( (*vector) == NULL )
    {
      LALZCreateVector ( status, vector, length );
    }
  else if ( length == 0 )
    {
      LALZDestroyVector ( status, vector );
    }
  else
    {
      /* 
       * Reallocate storage 
       * Test that storage is properly allocated. Can't handle with ASSERT
       * since we need to de-allocate structure pointer before an error return
       */

      /* 
         NOTE: LALRealloc() free()s the pointer that's been passed in.  This
         is different from the documented behavior of realloc(3) on Darwin,
         although the realloc(3) on Darwin actually *does* free() the memory.
         Ignore what Darwin's man pages say about realloc() and reallocf() 
       */
      p = LALRealloc( (*vector)->data, length*sizeof( COMPLEX16 ));

      if (p == NULL)
        {
          /* FIXME: question -- do I need to free (*vector)->data here? */
          LALFree( (*vector)->data );
          ABORT( status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC );
        }
    
      (*vector)->data = p;
      (*vector)->length = length;	/* Set length if storage allocated */
    }

  RETURN( status );
}
















/* <lalVerbatim file="VectorFactoriesD"> */
void LALZDestroyVector ( LALStatus *status, COMPLEX16Vector **vector )
{ /* </lalVerbatim> */
  /* 
   * Initialize status
   */

  INITSTATUS( status, "LALZDestroyVector", VECTORFACTORIESC );	
      
  /* 
   * Check vector: is it non-NULL?
   */

  ASSERT( vector != NULL, status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR );

  /* 
   * Check vector: does it point to non-NULL?
   */

  ASSERT( *vector != NULL, status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR );

  /*
   * Check data in vector: does it point to non-NULL
   */

  ASSERT( (*vector)->data != NULL, status,
          AVFACTORIESH_EDPTR, AVFACTORIESH_MSGEDPTR );

  /* Ok, now let's free allocated storage */

  LALFree( (*vector)->data ); /* free allocated data */
  LALFree( *vector );	/* free vector struct itself */

  *vector = NULL;		/* make sure we don't point to freed struct */

  RETURN( status );
}


















/* <lalVerbatim file="VectorFactoriesD"> */
void LALCCreateVector ( LALStatus *status, COMPLEX8Vector **vector, UINT4 length ) 
{ /* </lalVerbatim> */
  /* 
   * Initialize status structure
   */

  INITSTATUS( status, "LALCCreateVector", VECTORFACTORIESC );	
      
  /* Check sequence length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT( length > 0, status, AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH );

  /* 
   * Check return structure: If return pointer does not point to a
   *    valid pointer then report an error 
   */

  ASSERT( vector != NULL, status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR );

  ASSERT( *vector == NULL, status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR );

  /*
   * Allocate pointer
   */

  *vector = ( COMPLEX8Vector * ) LALMalloc( sizeof( COMPLEX8Vector ) );
  if ( NULL == *vector )
  {
    ABORT( status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC );
  }

  (*vector)->length = 0;	/* length 0 until storage allocated */
  (*vector)->data   = NULL;	/* NULL data until allocated */

  /* 
   * Allocate storage 
   * Test that storage is properly allocated. Can't handle with ASSERT
   * since we need to de-allocate structure pointer before an error return
   */

  (*vector)->data = ( COMPLEX8 * ) LALMalloc( length*sizeof( COMPLEX8 ) );

  if ( NULL == (*vector)->data )
  {
    /* Must free storage pointed to by *vector */
    LALFree( *vector );
    ABORT( status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC );
  }
  (*vector)->length = length;	/* Set length if storage allocated */

  /* We be done: Normal exit */

  RETURN( status );
}


















/* <lalVerbatim file="VectorFactoriesD"> */
void LALCResizeVector ( LALStatus *status, COMPLEX8Vector **vector, UINT4 length ) 
{ /* </lalVerbatim> */
  COMPLEX8 *p; /* temporary pointer */

  /* 
   * Initialize status structure
   */
  INITSTATUS( status, "LALCResizeVector", VECTORFACTORIESC );	

  ASSERT ( vector != NULL, status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR );
      
  /* Want this to behave like realloc(3), i.e.
   * *vector == NULL => create a new vector 
   * length == 0 => destroy the vector 
   * otherwise => resize given vector 
   */
  if ( (*vector) == NULL )
    {
      LALCCreateVector ( status, vector, length );
    }
  else if ( length == 0 )
    {
      LALCDestroyVector ( status, vector );
    }
  else
    {
      /* 
       * Reallocate storage 
       * Test that storage is properly allocated. Can't handle with ASSERT
       * since we need to de-allocate structure pointer before an error return
       */

      /* 
         NOTE: LALRealloc() free()s the pointer that's been passed in.  This
         is different from the documented behavior of realloc(3) on Darwin,
         although the realloc(3) on Darwin actually *does* free() the memory.
         Ignore what Darwin's man pages say about realloc() and reallocf() 
       */
      p = LALRealloc( (*vector)->data, length*sizeof( COMPLEX8 ));

      if (p == NULL)
        {
          /* FIXME: question -- do I need to free (*vector)->data here? */
          LALFree( (*vector)->data );
          ABORT( status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC );
        }
    
      (*vector)->data = p;
      (*vector)->length = length;	/* Set length if storage allocated */
    }

  RETURN( status );
}
















/* <lalVerbatim file="VectorFactoriesD"> */
void LALCDestroyVector ( LALStatus *status, COMPLEX8Vector **vector )
{ /* </lalVerbatim> */
  /* 
   * Initialize status
   */

  INITSTATUS( status, "LALCDestroyVector", VECTORFACTORIESC );	
      
  /* 
   * Check vector: is it non-NULL?
   */

  ASSERT( vector != NULL, status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR );

  /* 
   * Check vector: does it point to non-NULL?
   */

  ASSERT( *vector != NULL, status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR );

  /*
   * Check data in vector: does it point to non-NULL
   */

  ASSERT( (*vector)->data != NULL, status,
          AVFACTORIESH_EDPTR, AVFACTORIESH_MSGEDPTR );

  /* Ok, now let's free allocated storage */

  LALFree( (*vector)->data ); /* free allocated data */
  LALFree( *vector );	/* free vector struct itself */

  *vector = NULL;		/* make sure we don't point to freed struct */

  RETURN( status );
}


















/* <lalVerbatim file="VectorFactoriesD"> */
void LALDCreateVector ( LALStatus *status, REAL8Vector **vector, UINT4 length ) 
{ /* </lalVerbatim> */
  /* 
   * Initialize status structure
   */

  INITSTATUS( status, "LALDCreateVector", VECTORFACTORIESC );	
      
  /* Check sequence length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT( length > 0, status, AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH );

  /* 
   * Check return structure: If return pointer does not point to a
   *    valid pointer then report an error 
   */

  ASSERT( vector != NULL, status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR );

  ASSERT( *vector == NULL, status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR );

  /*
   * Allocate pointer
   */

  *vector = ( REAL8Vector * ) LALMalloc( sizeof( REAL8Vector ) );
  if ( NULL == *vector )
  {
    ABORT( status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC );
  }

  (*vector)->length = 0;	/* length 0 until storage allocated */
  (*vector)->data   = NULL;	/* NULL data until allocated */

  /* 
   * Allocate storage 
   * Test that storage is properly allocated. Can't handle with ASSERT
   * since we need to de-allocate structure pointer before an error return
   */

  (*vector)->data = ( REAL8 * ) LALMalloc( length*sizeof( REAL8 ) );

  if ( NULL == (*vector)->data )
  {
    /* Must free storage pointed to by *vector */
    LALFree( *vector );
    ABORT( status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC );
  }
  (*vector)->length = length;	/* Set length if storage allocated */

  /* We be done: Normal exit */

  RETURN( status );
}


















/* <lalVerbatim file="VectorFactoriesD"> */
void LALDResizeVector ( LALStatus *status, REAL8Vector **vector, UINT4 length ) 
{ /* </lalVerbatim> */
  REAL8 *p; /* temporary pointer */

  /* 
   * Initialize status structure
   */
  INITSTATUS( status, "LALDResizeVector", VECTORFACTORIESC );	

  ASSERT ( vector != NULL, status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR );
      
  /* Want this to behave like realloc(3), i.e.
   * *vector == NULL => create a new vector 
   * length == 0 => destroy the vector 
   * otherwise => resize given vector 
   */
  if ( (*vector) == NULL )
    {
      LALDCreateVector ( status, vector, length );
    }
  else if ( length == 0 )
    {
      LALDDestroyVector ( status, vector );
    }
  else
    {
      /* 
       * Reallocate storage 
       * Test that storage is properly allocated. Can't handle with ASSERT
       * since we need to de-allocate structure pointer before an error return
       */

      /* 
         NOTE: LALRealloc() free()s the pointer that's been passed in.  This
         is different from the documented behavior of realloc(3) on Darwin,
         although the realloc(3) on Darwin actually *does* free() the memory.
         Ignore what Darwin's man pages say about realloc() and reallocf() 
       */
      p = LALRealloc( (*vector)->data, length*sizeof( REAL8 ));

      if (p == NULL)
        {
          /* FIXME: question -- do I need to free (*vector)->data here? */
          LALFree( (*vector)->data );
          ABORT( status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC );
        }
    
      (*vector)->data = p;
      (*vector)->length = length;	/* Set length if storage allocated */
    }

  RETURN( status );
}
















/* <lalVerbatim file="VectorFactoriesD"> */
void LALDDestroyVector ( LALStatus *status, REAL8Vector **vector )
{ /* </lalVerbatim> */
  /* 
   * Initialize status
   */

  INITSTATUS( status, "LALDDestroyVector", VECTORFACTORIESC );	
      
  /* 
   * Check vector: is it non-NULL?
   */

  ASSERT( vector != NULL, status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR );

  /* 
   * Check vector: does it point to non-NULL?
   */

  ASSERT( *vector != NULL, status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR );

  /*
   * Check data in vector: does it point to non-NULL
   */

  ASSERT( (*vector)->data != NULL, status,
          AVFACTORIESH_EDPTR, AVFACTORIESH_MSGEDPTR );

  /* Ok, now let's free allocated storage */

  LALFree( (*vector)->data ); /* free allocated data */
  LALFree( *vector );	/* free vector struct itself */

  *vector = NULL;		/* make sure we don't point to freed struct */

  RETURN( status );
}


















/* <lalVerbatim file="VectorFactoriesD"> */
void LALSCreateVector ( LALStatus *status, REAL4Vector **vector, UINT4 length ) 
{ /* </lalVerbatim> */
  /* 
   * Initialize status structure
   */

  INITSTATUS( status, "LALSCreateVector", VECTORFACTORIESC );	
      
  /* Check sequence length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT( length > 0, status, AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH );

  /* 
   * Check return structure: If return pointer does not point to a
   *    valid pointer then report an error 
   */

  ASSERT( vector != NULL, status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR );

  ASSERT( *vector == NULL, status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR );

  /*
   * Allocate pointer
   */

  *vector = ( REAL4Vector * ) LALMalloc( sizeof( REAL4Vector ) );
  if ( NULL == *vector )
  {
    ABORT( status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC );
  }

  (*vector)->length = 0;	/* length 0 until storage allocated */
  (*vector)->data   = NULL;	/* NULL data until allocated */

  /* 
   * Allocate storage 
   * Test that storage is properly allocated. Can't handle with ASSERT
   * since we need to de-allocate structure pointer before an error return
   */

  (*vector)->data = ( REAL4 * ) LALMalloc( length*sizeof( REAL4 ) );

  if ( NULL == (*vector)->data )
  {
    /* Must free storage pointed to by *vector */
    LALFree( *vector );
    ABORT( status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC );
  }
  (*vector)->length = length;	/* Set length if storage allocated */

  /* We be done: Normal exit */

  RETURN( status );
}


















/* <lalVerbatim file="VectorFactoriesD"> */
void LALSResizeVector ( LALStatus *status, REAL4Vector **vector, UINT4 length ) 
{ /* </lalVerbatim> */
  REAL4 *p; /* temporary pointer */

  /* 
   * Initialize status structure
   */
  INITSTATUS( status, "LALSResizeVector", VECTORFACTORIESC );	

  ASSERT ( vector != NULL, status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR );
      
  /* Want this to behave like realloc(3), i.e.
   * *vector == NULL => create a new vector 
   * length == 0 => destroy the vector 
   * otherwise => resize given vector 
   */
  if ( (*vector) == NULL )
    {
      LALSCreateVector ( status, vector, length );
    }
  else if ( length == 0 )
    {
      LALSDestroyVector ( status, vector );
    }
  else
    {
      /* 
       * Reallocate storage 
       * Test that storage is properly allocated. Can't handle with ASSERT
       * since we need to de-allocate structure pointer before an error return
       */

      /* 
         NOTE: LALRealloc() free()s the pointer that's been passed in.  This
         is different from the documented behavior of realloc(3) on Darwin,
         although the realloc(3) on Darwin actually *does* free() the memory.
         Ignore what Darwin's man pages say about realloc() and reallocf() 
       */
      p = LALRealloc( (*vector)->data, length*sizeof( REAL4 ));

      if (p == NULL)
        {
          /* FIXME: question -- do I need to free (*vector)->data here? */
          LALFree( (*vector)->data );
          ABORT( status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC );
        }
    
      (*vector)->data = p;
      (*vector)->length = length;	/* Set length if storage allocated */
    }

  RETURN( status );
}
















/* <lalVerbatim file="VectorFactoriesD"> */
void LALSDestroyVector ( LALStatus *status, REAL4Vector **vector )
{ /* </lalVerbatim> */
  /* 
   * Initialize status
   */

  INITSTATUS( status, "LALSDestroyVector", VECTORFACTORIESC );	
      
  /* 
   * Check vector: is it non-NULL?
   */

  ASSERT( vector != NULL, status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR );

  /* 
   * Check vector: does it point to non-NULL?
   */

  ASSERT( *vector != NULL, status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR );

  /*
   * Check data in vector: does it point to non-NULL
   */

  ASSERT( (*vector)->data != NULL, status,
          AVFACTORIESH_EDPTR, AVFACTORIESH_MSGEDPTR );

  /* Ok, now let's free allocated storage */

  LALFree( (*vector)->data ); /* free allocated data */
  LALFree( *vector );	/* free vector struct itself */

  *vector = NULL;		/* make sure we don't point to freed struct */

  RETURN( status );
}


















/* <lalVerbatim file="VectorFactoriesD"> */
void LALI2CreateVector ( LALStatus *status, INT2Vector **vector, UINT4 length ) 
{ /* </lalVerbatim> */
  /* 
   * Initialize status structure
   */

  INITSTATUS( status, "LALI2CreateVector", VECTORFACTORIESC );	
      
  /* Check sequence length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT( length > 0, status, AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH );

  /* 
   * Check return structure: If return pointer does not point to a
   *    valid pointer then report an error 
   */

  ASSERT( vector != NULL, status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR );

  ASSERT( *vector == NULL, status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR );

  /*
   * Allocate pointer
   */

  *vector = ( INT2Vector * ) LALMalloc( sizeof( INT2Vector ) );
  if ( NULL == *vector )
  {
    ABORT( status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC );
  }

  (*vector)->length = 0;	/* length 0 until storage allocated */
  (*vector)->data   = NULL;	/* NULL data until allocated */

  /* 
   * Allocate storage 
   * Test that storage is properly allocated. Can't handle with ASSERT
   * since we need to de-allocate structure pointer before an error return
   */

  (*vector)->data = ( INT2 * ) LALMalloc( length*sizeof( INT2 ) );

  if ( NULL == (*vector)->data )
  {
    /* Must free storage pointed to by *vector */
    LALFree( *vector );
    ABORT( status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC );
  }
  (*vector)->length = length;	/* Set length if storage allocated */

  /* We be done: Normal exit */

  RETURN( status );
}


















/* <lalVerbatim file="VectorFactoriesD"> */
void LALI2ResizeVector ( LALStatus *status, INT2Vector **vector, UINT4 length ) 
{ /* </lalVerbatim> */
  INT2 *p; /* temporary pointer */

  /* 
   * Initialize status structure
   */
  INITSTATUS( status, "LALI2ResizeVector", VECTORFACTORIESC );	

  ASSERT ( vector != NULL, status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR );
      
  /* Want this to behave like realloc(3), i.e.
   * *vector == NULL => create a new vector 
   * length == 0 => destroy the vector 
   * otherwise => resize given vector 
   */
  if ( (*vector) == NULL )
    {
      LALI2CreateVector ( status, vector, length );
    }
  else if ( length == 0 )
    {
      LALI2DestroyVector ( status, vector );
    }
  else
    {
      /* 
       * Reallocate storage 
       * Test that storage is properly allocated. Can't handle with ASSERT
       * since we need to de-allocate structure pointer before an error return
       */

      /* 
         NOTE: LALRealloc() free()s the pointer that's been passed in.  This
         is different from the documented behavior of realloc(3) on Darwin,
         although the realloc(3) on Darwin actually *does* free() the memory.
         Ignore what Darwin's man pages say about realloc() and reallocf() 
       */
      p = LALRealloc( (*vector)->data, length*sizeof( INT2 ));

      if (p == NULL)
        {
          /* FIXME: question -- do I need to free (*vector)->data here? */
          LALFree( (*vector)->data );
          ABORT( status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC );
        }
    
      (*vector)->data = p;
      (*vector)->length = length;	/* Set length if storage allocated */
    }

  RETURN( status );
}
















/* <lalVerbatim file="VectorFactoriesD"> */
void LALI2DestroyVector ( LALStatus *status, INT2Vector **vector )
{ /* </lalVerbatim> */
  /* 
   * Initialize status
   */

  INITSTATUS( status, "LALI2DestroyVector", VECTORFACTORIESC );	
      
  /* 
   * Check vector: is it non-NULL?
   */

  ASSERT( vector != NULL, status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR );

  /* 
   * Check vector: does it point to non-NULL?
   */

  ASSERT( *vector != NULL, status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR );

  /*
   * Check data in vector: does it point to non-NULL
   */

  ASSERT( (*vector)->data != NULL, status,
          AVFACTORIESH_EDPTR, AVFACTORIESH_MSGEDPTR );

  /* Ok, now let's free allocated storage */

  LALFree( (*vector)->data ); /* free allocated data */
  LALFree( *vector );	/* free vector struct itself */

  *vector = NULL;		/* make sure we don't point to freed struct */

  RETURN( status );
}


















/* <lalVerbatim file="VectorFactoriesD"> */
void LALI4CreateVector ( LALStatus *status, INT4Vector **vector, UINT4 length ) 
{ /* </lalVerbatim> */
  /* 
   * Initialize status structure
   */

  INITSTATUS( status, "LALI4CreateVector", VECTORFACTORIESC );	
      
  /* Check sequence length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT( length > 0, status, AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH );

  /* 
   * Check return structure: If return pointer does not point to a
   *    valid pointer then report an error 
   */

  ASSERT( vector != NULL, status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR );

  ASSERT( *vector == NULL, status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR );

  /*
   * Allocate pointer
   */

  *vector = ( INT4Vector * ) LALMalloc( sizeof( INT4Vector ) );
  if ( NULL == *vector )
  {
    ABORT( status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC );
  }

  (*vector)->length = 0;	/* length 0 until storage allocated */
  (*vector)->data   = NULL;	/* NULL data until allocated */

  /* 
   * Allocate storage 
   * Test that storage is properly allocated. Can't handle with ASSERT
   * since we need to de-allocate structure pointer before an error return
   */

  (*vector)->data = ( INT4 * ) LALMalloc( length*sizeof( INT4 ) );

  if ( NULL == (*vector)->data )
  {
    /* Must free storage pointed to by *vector */
    LALFree( *vector );
    ABORT( status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC );
  }
  (*vector)->length = length;	/* Set length if storage allocated */

  /* We be done: Normal exit */

  RETURN( status );
}


















/* <lalVerbatim file="VectorFactoriesD"> */
void LALI4ResizeVector ( LALStatus *status, INT4Vector **vector, UINT4 length ) 
{ /* </lalVerbatim> */
  INT4 *p; /* temporary pointer */

  /* 
   * Initialize status structure
   */
  INITSTATUS( status, "LALI4ResizeVector", VECTORFACTORIESC );	

  ASSERT ( vector != NULL, status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR );
      
  /* Want this to behave like realloc(3), i.e.
   * *vector == NULL => create a new vector 
   * length == 0 => destroy the vector 
   * otherwise => resize given vector 
   */
  if ( (*vector) == NULL )
    {
      LALI4CreateVector ( status, vector, length );
    }
  else if ( length == 0 )
    {
      LALI4DestroyVector ( status, vector );
    }
  else
    {
      /* 
       * Reallocate storage 
       * Test that storage is properly allocated. Can't handle with ASSERT
       * since we need to de-allocate structure pointer before an error return
       */

      /* 
         NOTE: LALRealloc() free()s the pointer that's been passed in.  This
         is different from the documented behavior of realloc(3) on Darwin,
         although the realloc(3) on Darwin actually *does* free() the memory.
         Ignore what Darwin's man pages say about realloc() and reallocf() 
       */
      p = LALRealloc( (*vector)->data, length*sizeof( INT4 ));

      if (p == NULL)
        {
          /* FIXME: question -- do I need to free (*vector)->data here? */
          LALFree( (*vector)->data );
          ABORT( status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC );
        }
    
      (*vector)->data = p;
      (*vector)->length = length;	/* Set length if storage allocated */
    }

  RETURN( status );
}
















/* <lalVerbatim file="VectorFactoriesD"> */
void LALI4DestroyVector ( LALStatus *status, INT4Vector **vector )
{ /* </lalVerbatim> */
  /* 
   * Initialize status
   */

  INITSTATUS( status, "LALI4DestroyVector", VECTORFACTORIESC );	
      
  /* 
   * Check vector: is it non-NULL?
   */

  ASSERT( vector != NULL, status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR );

  /* 
   * Check vector: does it point to non-NULL?
   */

  ASSERT( *vector != NULL, status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR );

  /*
   * Check data in vector: does it point to non-NULL
   */

  ASSERT( (*vector)->data != NULL, status,
          AVFACTORIESH_EDPTR, AVFACTORIESH_MSGEDPTR );

  /* Ok, now let's free allocated storage */

  LALFree( (*vector)->data ); /* free allocated data */
  LALFree( *vector );	/* free vector struct itself */

  *vector = NULL;		/* make sure we don't point to freed struct */

  RETURN( status );
}


















/* <lalVerbatim file="VectorFactoriesD"> */
void LALI8CreateVector ( LALStatus *status, INT8Vector **vector, UINT4 length ) 
{ /* </lalVerbatim> */
  /* 
   * Initialize status structure
   */

  INITSTATUS( status, "LALI8CreateVector", VECTORFACTORIESC );	
      
  /* Check sequence length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT( length > 0, status, AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH );

  /* 
   * Check return structure: If return pointer does not point to a
   *    valid pointer then report an error 
   */

  ASSERT( vector != NULL, status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR );

  ASSERT( *vector == NULL, status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR );

  /*
   * Allocate pointer
   */

  *vector = ( INT8Vector * ) LALMalloc( sizeof( INT8Vector ) );
  if ( NULL == *vector )
  {
    ABORT( status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC );
  }

  (*vector)->length = 0;	/* length 0 until storage allocated */
  (*vector)->data   = NULL;	/* NULL data until allocated */

  /* 
   * Allocate storage 
   * Test that storage is properly allocated. Can't handle with ASSERT
   * since we need to de-allocate structure pointer before an error return
   */

  (*vector)->data = ( INT8 * ) LALMalloc( length*sizeof( INT8 ) );

  if ( NULL == (*vector)->data )
  {
    /* Must free storage pointed to by *vector */
    LALFree( *vector );
    ABORT( status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC );
  }
  (*vector)->length = length;	/* Set length if storage allocated */

  /* We be done: Normal exit */

  RETURN( status );
}


















/* <lalVerbatim file="VectorFactoriesD"> */
void LALI8ResizeVector ( LALStatus *status, INT8Vector **vector, UINT4 length ) 
{ /* </lalVerbatim> */
  INT8 *p; /* temporary pointer */

  /* 
   * Initialize status structure
   */
  INITSTATUS( status, "LALI8ResizeVector", VECTORFACTORIESC );	

  ASSERT ( vector != NULL, status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR );
      
  /* Want this to behave like realloc(3), i.e.
   * *vector == NULL => create a new vector 
   * length == 0 => destroy the vector 
   * otherwise => resize given vector 
   */
  if ( (*vector) == NULL )
    {
      LALI8CreateVector ( status, vector, length );
    }
  else if ( length == 0 )
    {
      LALI8DestroyVector ( status, vector );
    }
  else
    {
      /* 
       * Reallocate storage 
       * Test that storage is properly allocated. Can't handle with ASSERT
       * since we need to de-allocate structure pointer before an error return
       */

      /* 
         NOTE: LALRealloc() free()s the pointer that's been passed in.  This
         is different from the documented behavior of realloc(3) on Darwin,
         although the realloc(3) on Darwin actually *does* free() the memory.
         Ignore what Darwin's man pages say about realloc() and reallocf() 
       */
      p = LALRealloc( (*vector)->data, length*sizeof( INT8 ));

      if (p == NULL)
        {
          /* FIXME: question -- do I need to free (*vector)->data here? */
          LALFree( (*vector)->data );
          ABORT( status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC );
        }
    
      (*vector)->data = p;
      (*vector)->length = length;	/* Set length if storage allocated */
    }

  RETURN( status );
}
















/* <lalVerbatim file="VectorFactoriesD"> */
void LALI8DestroyVector ( LALStatus *status, INT8Vector **vector )
{ /* </lalVerbatim> */
  /* 
   * Initialize status
   */

  INITSTATUS( status, "LALI8DestroyVector", VECTORFACTORIESC );	
      
  /* 
   * Check vector: is it non-NULL?
   */

  ASSERT( vector != NULL, status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR );

  /* 
   * Check vector: does it point to non-NULL?
   */

  ASSERT( *vector != NULL, status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR );

  /*
   * Check data in vector: does it point to non-NULL
   */

  ASSERT( (*vector)->data != NULL, status,
          AVFACTORIESH_EDPTR, AVFACTORIESH_MSGEDPTR );

  /* Ok, now let's free allocated storage */

  LALFree( (*vector)->data ); /* free allocated data */
  LALFree( *vector );	/* free vector struct itself */

  *vector = NULL;		/* make sure we don't point to freed struct */

  RETURN( status );
}


















/* <lalVerbatim file="VectorFactoriesD"> */
void LALU2CreateVector ( LALStatus *status, UINT2Vector **vector, UINT4 length ) 
{ /* </lalVerbatim> */
  /* 
   * Initialize status structure
   */

  INITSTATUS( status, "LALU2CreateVector", VECTORFACTORIESC );	
      
  /* Check sequence length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT( length > 0, status, AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH );

  /* 
   * Check return structure: If return pointer does not point to a
   *    valid pointer then report an error 
   */

  ASSERT( vector != NULL, status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR );

  ASSERT( *vector == NULL, status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR );

  /*
   * Allocate pointer
   */

  *vector = ( UINT2Vector * ) LALMalloc( sizeof( UINT2Vector ) );
  if ( NULL == *vector )
  {
    ABORT( status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC );
  }

  (*vector)->length = 0;	/* length 0 until storage allocated */
  (*vector)->data   = NULL;	/* NULL data until allocated */

  /* 
   * Allocate storage 
   * Test that storage is properly allocated. Can't handle with ASSERT
   * since we need to de-allocate structure pointer before an error return
   */

  (*vector)->data = ( UINT2 * ) LALMalloc( length*sizeof( UINT2 ) );

  if ( NULL == (*vector)->data )
  {
    /* Must free storage pointed to by *vector */
    LALFree( *vector );
    ABORT( status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC );
  }
  (*vector)->length = length;	/* Set length if storage allocated */

  /* We be done: Normal exit */

  RETURN( status );
}


















/* <lalVerbatim file="VectorFactoriesD"> */
void LALU2ResizeVector ( LALStatus *status, UINT2Vector **vector, UINT4 length ) 
{ /* </lalVerbatim> */
  UINT2 *p; /* temporary pointer */

  /* 
   * Initialize status structure
   */
  INITSTATUS( status, "LALU2ResizeVector", VECTORFACTORIESC );	

  ASSERT ( vector != NULL, status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR );
      
  /* Want this to behave like realloc(3), i.e.
   * *vector == NULL => create a new vector 
   * length == 0 => destroy the vector 
   * otherwise => resize given vector 
   */
  if ( (*vector) == NULL )
    {
      LALU2CreateVector ( status, vector, length );
    }
  else if ( length == 0 )
    {
      LALU2DestroyVector ( status, vector );
    }
  else
    {
      /* 
       * Reallocate storage 
       * Test that storage is properly allocated. Can't handle with ASSERT
       * since we need to de-allocate structure pointer before an error return
       */

      /* 
         NOTE: LALRealloc() free()s the pointer that's been passed in.  This
         is different from the documented behavior of realloc(3) on Darwin,
         although the realloc(3) on Darwin actually *does* free() the memory.
         Ignore what Darwin's man pages say about realloc() and reallocf() 
       */
      p = LALRealloc( (*vector)->data, length*sizeof( UINT2 ));

      if (p == NULL)
        {
          /* FIXME: question -- do I need to free (*vector)->data here? */
          LALFree( (*vector)->data );
          ABORT( status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC );
        }
    
      (*vector)->data = p;
      (*vector)->length = length;	/* Set length if storage allocated */
    }

  RETURN( status );
}
















/* <lalVerbatim file="VectorFactoriesD"> */
void LALU2DestroyVector ( LALStatus *status, UINT2Vector **vector )
{ /* </lalVerbatim> */
  /* 
   * Initialize status
   */

  INITSTATUS( status, "LALU2DestroyVector", VECTORFACTORIESC );	
      
  /* 
   * Check vector: is it non-NULL?
   */

  ASSERT( vector != NULL, status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR );

  /* 
   * Check vector: does it point to non-NULL?
   */

  ASSERT( *vector != NULL, status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR );

  /*
   * Check data in vector: does it point to non-NULL
   */

  ASSERT( (*vector)->data != NULL, status,
          AVFACTORIESH_EDPTR, AVFACTORIESH_MSGEDPTR );

  /* Ok, now let's free allocated storage */

  LALFree( (*vector)->data ); /* free allocated data */
  LALFree( *vector );	/* free vector struct itself */

  *vector = NULL;		/* make sure we don't point to freed struct */

  RETURN( status );
}


















/* <lalVerbatim file="VectorFactoriesD"> */
void LALU4CreateVector ( LALStatus *status, UINT4Vector **vector, UINT4 length ) 
{ /* </lalVerbatim> */
  /* 
   * Initialize status structure
   */

  INITSTATUS( status, "LALU4CreateVector", VECTORFACTORIESC );	
      
  /* Check sequence length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT( length > 0, status, AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH );

  /* 
   * Check return structure: If return pointer does not point to a
   *    valid pointer then report an error 
   */

  ASSERT( vector != NULL, status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR );

  ASSERT( *vector == NULL, status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR );

  /*
   * Allocate pointer
   */

  *vector = ( UINT4Vector * ) LALMalloc( sizeof( UINT4Vector ) );
  if ( NULL == *vector )
  {
    ABORT( status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC );
  }

  (*vector)->length = 0;	/* length 0 until storage allocated */
  (*vector)->data   = NULL;	/* NULL data until allocated */

  /* 
   * Allocate storage 
   * Test that storage is properly allocated. Can't handle with ASSERT
   * since we need to de-allocate structure pointer before an error return
   */

  (*vector)->data = ( UINT4 * ) LALMalloc( length*sizeof( UINT4 ) );

  if ( NULL == (*vector)->data )
  {
    /* Must free storage pointed to by *vector */
    LALFree( *vector );
    ABORT( status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC );
  }
  (*vector)->length = length;	/* Set length if storage allocated */

  /* We be done: Normal exit */

  RETURN( status );
}


















/* <lalVerbatim file="VectorFactoriesD"> */
void LALU4ResizeVector ( LALStatus *status, UINT4Vector **vector, UINT4 length ) 
{ /* </lalVerbatim> */
  UINT4 *p; /* temporary pointer */

  /* 
   * Initialize status structure
   */
  INITSTATUS( status, "LALU4ResizeVector", VECTORFACTORIESC );	

  ASSERT ( vector != NULL, status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR );
      
  /* Want this to behave like realloc(3), i.e.
   * *vector == NULL => create a new vector 
   * length == 0 => destroy the vector 
   * otherwise => resize given vector 
   */
  if ( (*vector) == NULL )
    {
      LALU4CreateVector ( status, vector, length );
    }
  else if ( length == 0 )
    {
      LALU4DestroyVector ( status, vector );
    }
  else
    {
      /* 
       * Reallocate storage 
       * Test that storage is properly allocated. Can't handle with ASSERT
       * since we need to de-allocate structure pointer before an error return
       */

      /* 
         NOTE: LALRealloc() free()s the pointer that's been passed in.  This
         is different from the documented behavior of realloc(3) on Darwin,
         although the realloc(3) on Darwin actually *does* free() the memory.
         Ignore what Darwin's man pages say about realloc() and reallocf() 
       */
      p = LALRealloc( (*vector)->data, length*sizeof( UINT4 ));

      if (p == NULL)
        {
          /* FIXME: question -- do I need to free (*vector)->data here? */
          LALFree( (*vector)->data );
          ABORT( status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC );
        }
    
      (*vector)->data = p;
      (*vector)->length = length;	/* Set length if storage allocated */
    }

  RETURN( status );
}
















/* <lalVerbatim file="VectorFactoriesD"> */
void LALU4DestroyVector ( LALStatus *status, UINT4Vector **vector )
{ /* </lalVerbatim> */
  /* 
   * Initialize status
   */

  INITSTATUS( status, "LALU4DestroyVector", VECTORFACTORIESC );	
      
  /* 
   * Check vector: is it non-NULL?
   */

  ASSERT( vector != NULL, status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR );

  /* 
   * Check vector: does it point to non-NULL?
   */

  ASSERT( *vector != NULL, status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR );

  /*
   * Check data in vector: does it point to non-NULL
   */

  ASSERT( (*vector)->data != NULL, status,
          AVFACTORIESH_EDPTR, AVFACTORIESH_MSGEDPTR );

  /* Ok, now let's free allocated storage */

  LALFree( (*vector)->data ); /* free allocated data */
  LALFree( *vector );	/* free vector struct itself */

  *vector = NULL;		/* make sure we don't point to freed struct */

  RETURN( status );
}


















/* <lalVerbatim file="VectorFactoriesD"> */
void LALU8CreateVector ( LALStatus *status, UINT8Vector **vector, UINT4 length ) 
{ /* </lalVerbatim> */
  /* 
   * Initialize status structure
   */

  INITSTATUS( status, "LALU8CreateVector", VECTORFACTORIESC );	
      
  /* Check sequence length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT( length > 0, status, AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH );

  /* 
   * Check return structure: If return pointer does not point to a
   *    valid pointer then report an error 
   */

  ASSERT( vector != NULL, status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR );

  ASSERT( *vector == NULL, status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR );

  /*
   * Allocate pointer
   */

  *vector = ( UINT8Vector * ) LALMalloc( sizeof( UINT8Vector ) );
  if ( NULL == *vector )
  {
    ABORT( status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC );
  }

  (*vector)->length = 0;	/* length 0 until storage allocated */
  (*vector)->data   = NULL;	/* NULL data until allocated */

  /* 
   * Allocate storage 
   * Test that storage is properly allocated. Can't handle with ASSERT
   * since we need to de-allocate structure pointer before an error return
   */

  (*vector)->data = ( UINT8 * ) LALMalloc( length*sizeof( UINT8 ) );

  if ( NULL == (*vector)->data )
  {
    /* Must free storage pointed to by *vector */
    LALFree( *vector );
    ABORT( status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC );
  }
  (*vector)->length = length;	/* Set length if storage allocated */

  /* We be done: Normal exit */

  RETURN( status );
}


















/* <lalVerbatim file="VectorFactoriesD"> */
void LALU8ResizeVector ( LALStatus *status, UINT8Vector **vector, UINT4 length ) 
{ /* </lalVerbatim> */
  UINT8 *p; /* temporary pointer */

  /* 
   * Initialize status structure
   */
  INITSTATUS( status, "LALU8ResizeVector", VECTORFACTORIESC );	

  ASSERT ( vector != NULL, status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR );
      
  /* Want this to behave like realloc(3), i.e.
   * *vector == NULL => create a new vector 
   * length == 0 => destroy the vector 
   * otherwise => resize given vector 
   */
  if ( (*vector) == NULL )
    {
      LALU8CreateVector ( status, vector, length );
    }
  else if ( length == 0 )
    {
      LALU8DestroyVector ( status, vector );
    }
  else
    {
      /* 
       * Reallocate storage 
       * Test that storage is properly allocated. Can't handle with ASSERT
       * since we need to de-allocate structure pointer before an error return
       */

      /* 
         NOTE: LALRealloc() free()s the pointer that's been passed in.  This
         is different from the documented behavior of realloc(3) on Darwin,
         although the realloc(3) on Darwin actually *does* free() the memory.
         Ignore what Darwin's man pages say about realloc() and reallocf() 
       */
      p = LALRealloc( (*vector)->data, length*sizeof( UINT8 ));

      if (p == NULL)
        {
          /* FIXME: question -- do I need to free (*vector)->data here? */
          LALFree( (*vector)->data );
          ABORT( status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC );
        }
    
      (*vector)->data = p;
      (*vector)->length = length;	/* Set length if storage allocated */
    }

  RETURN( status );
}
















/* <lalVerbatim file="VectorFactoriesD"> */
void LALU8DestroyVector ( LALStatus *status, UINT8Vector **vector )
{ /* </lalVerbatim> */
  /* 
   * Initialize status
   */

  INITSTATUS( status, "LALU8DestroyVector", VECTORFACTORIESC );	
      
  /* 
   * Check vector: is it non-NULL?
   */

  ASSERT( vector != NULL, status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR );

  /* 
   * Check vector: does it point to non-NULL?
   */

  ASSERT( *vector != NULL, status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR );

  /*
   * Check data in vector: does it point to non-NULL
   */

  ASSERT( (*vector)->data != NULL, status,
          AVFACTORIESH_EDPTR, AVFACTORIESH_MSGEDPTR );

  /* Ok, now let's free allocated storage */

  LALFree( (*vector)->data ); /* free allocated data */
  LALFree( *vector );	/* free vector struct itself */

  *vector = NULL;		/* make sure we don't point to freed struct */

  RETURN( status );
}


















/* <lalVerbatim file="VectorFactoriesD"> */
void LALCHARCreateVector ( LALStatus *status, CHARVector **vector, UINT4 length ) 
{ /* </lalVerbatim> */
  /* 
   * Initialize status structure
   */

  INITSTATUS( status, "LALCHARCreateVector", VECTORFACTORIESC );	
      
  /* Check sequence length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT( length > 0, status, AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH );

  /* 
   * Check return structure: If return pointer does not point to a
   *    valid pointer then report an error 
   */

  ASSERT( vector != NULL, status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR );

  ASSERT( *vector == NULL, status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR );

  /*
   * Allocate pointer
   */

  *vector = ( CHARVector * ) LALMalloc( sizeof( CHARVector ) );
  if ( NULL == *vector )
  {
    ABORT( status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC );
  }

  (*vector)->length = 0;	/* length 0 until storage allocated */
  (*vector)->data   = NULL;	/* NULL data until allocated */

  /* 
   * Allocate storage 
   * Test that storage is properly allocated. Can't handle with ASSERT
   * since we need to de-allocate structure pointer before an error return
   */

  (*vector)->data = ( CHAR * ) LALMalloc( length*sizeof( CHAR ) );

  if ( NULL == (*vector)->data )
  {
    /* Must free storage pointed to by *vector */
    LALFree( *vector );
    ABORT( status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC );
  }
  (*vector)->length = length;	/* Set length if storage allocated */

  /* We be done: Normal exit */

  RETURN( status );
}


















/* <lalVerbatim file="VectorFactoriesD"> */
void LALCHARResizeVector ( LALStatus *status, CHARVector **vector, UINT4 length ) 
{ /* </lalVerbatim> */
  CHAR *p; /* temporary pointer */

  /* 
   * Initialize status structure
   */
  INITSTATUS( status, "LALCHARResizeVector", VECTORFACTORIESC );	

  ASSERT ( vector != NULL, status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR );
      
  /* Want this to behave like realloc(3), i.e.
   * *vector == NULL => create a new vector 
   * length == 0 => destroy the vector 
   * otherwise => resize given vector 
   */
  if ( (*vector) == NULL )
    {
      LALCHARCreateVector ( status, vector, length );
    }
  else if ( length == 0 )
    {
      LALCHARDestroyVector ( status, vector );
    }
  else
    {
      /* 
       * Reallocate storage 
       * Test that storage is properly allocated. Can't handle with ASSERT
       * since we need to de-allocate structure pointer before an error return
       */

      /* 
         NOTE: LALRealloc() free()s the pointer that's been passed in.  This
         is different from the documented behavior of realloc(3) on Darwin,
         although the realloc(3) on Darwin actually *does* free() the memory.
         Ignore what Darwin's man pages say about realloc() and reallocf() 
       */
      p = LALRealloc( (*vector)->data, length*sizeof( CHAR ));

      if (p == NULL)
        {
          /* FIXME: question -- do I need to free (*vector)->data here? */
          LALFree( (*vector)->data );
          ABORT( status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC );
        }
    
      (*vector)->data = p;
      (*vector)->length = length;	/* Set length if storage allocated */
    }

  RETURN( status );
}
















/* <lalVerbatim file="VectorFactoriesD"> */
void LALCHARDestroyVector ( LALStatus *status, CHARVector **vector )
{ /* </lalVerbatim> */
  /* 
   * Initialize status
   */

  INITSTATUS( status, "LALCHARDestroyVector", VECTORFACTORIESC );	
      
  /* 
   * Check vector: is it non-NULL?
   */

  ASSERT( vector != NULL, status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR );

  /* 
   * Check vector: does it point to non-NULL?
   */

  ASSERT( *vector != NULL, status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR );

  /*
   * Check data in vector: does it point to non-NULL
   */

  ASSERT( (*vector)->data != NULL, status,
          AVFACTORIESH_EDPTR, AVFACTORIESH_MSGEDPTR );

  /* Ok, now let's free allocated storage */

  LALFree( (*vector)->data ); /* free allocated data */
  LALFree( *vector );	/* free vector struct itself */

  *vector = NULL;		/* make sure we don't point to freed struct */

  RETURN( status );
}


















/* <lalVerbatim file="VectorFactoriesD"> */
void LALCreateVector ( LALStatus *status, REAL4Vector **vector, UINT4 length ) 
{ /* </lalVerbatim> */
  /* 
   * Initialize status structure
   */

  INITSTATUS( status, "LALCreateVector", VECTORFACTORIESC );	
      
  /* Check sequence length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT( length > 0, status, AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH );

  /* 
   * Check return structure: If return pointer does not point to a
   *    valid pointer then report an error 
   */

  ASSERT( vector != NULL, status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR );

  ASSERT( *vector == NULL, status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR );

  /*
   * Allocate pointer
   */

  *vector = ( REAL4Vector * ) LALMalloc( sizeof( REAL4Vector ) );
  if ( NULL == *vector )
  {
    ABORT( status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC );
  }

  (*vector)->length = 0;	/* length 0 until storage allocated */
  (*vector)->data   = NULL;	/* NULL data until allocated */

  /* 
   * Allocate storage 
   * Test that storage is properly allocated. Can't handle with ASSERT
   * since we need to de-allocate structure pointer before an error return
   */

  (*vector)->data = ( REAL4 * ) LALMalloc( length*sizeof( REAL4 ) );

  if ( NULL == (*vector)->data )
  {
    /* Must free storage pointed to by *vector */
    LALFree( *vector );
    ABORT( status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC );
  }
  (*vector)->length = length;	/* Set length if storage allocated */

  /* We be done: Normal exit */

  RETURN( status );
}


















/* <lalVerbatim file="VectorFactoriesD"> */
void LALResizeVector ( LALStatus *status, REAL4Vector **vector, UINT4 length ) 
{ /* </lalVerbatim> */
  REAL4 *p; /* temporary pointer */

  /* 
   * Initialize status structure
   */
  INITSTATUS( status, "LALResizeVector", VECTORFACTORIESC );	

  ASSERT ( vector != NULL, status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR );
      
  /* Want this to behave like realloc(3), i.e.
   * *vector == NULL => create a new vector 
   * length == 0 => destroy the vector 
   * otherwise => resize given vector 
   */
  if ( (*vector) == NULL )
    {
      LALCreateVector ( status, vector, length );
    }
  else if ( length == 0 )
    {
      LALDestroyVector ( status, vector );
    }
  else
    {
      /* 
       * Reallocate storage 
       * Test that storage is properly allocated. Can't handle with ASSERT
       * since we need to de-allocate structure pointer before an error return
       */

      /* 
         NOTE: LALRealloc() free()s the pointer that's been passed in.  This
         is different from the documented behavior of realloc(3) on Darwin,
         although the realloc(3) on Darwin actually *does* free() the memory.
         Ignore what Darwin's man pages say about realloc() and reallocf() 
       */
      p = LALRealloc( (*vector)->data, length*sizeof( REAL4 ));

      if (p == NULL)
        {
          /* FIXME: question -- do I need to free (*vector)->data here? */
          LALFree( (*vector)->data );
          ABORT( status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC );
        }
    
      (*vector)->data = p;
      (*vector)->length = length;	/* Set length if storage allocated */
    }

  RETURN( status );
}
















/* <lalVerbatim file="VectorFactoriesD"> */
void LALDestroyVector ( LALStatus *status, REAL4Vector **vector )
{ /* </lalVerbatim> */
  /* 
   * Initialize status
   */

  INITSTATUS( status, "LALDestroyVector", VECTORFACTORIESC );	
      
  /* 
   * Check vector: is it non-NULL?
   */

  ASSERT( vector != NULL, status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR );

  /* 
   * Check vector: does it point to non-NULL?
   */

  ASSERT( *vector != NULL, status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR );

  /*
   * Check data in vector: does it point to non-NULL
   */

  ASSERT( (*vector)->data != NULL, status,
          AVFACTORIESH_EDPTR, AVFACTORIESH_MSGEDPTR );

  /* Ok, now let's free allocated storage */

  LALFree( (*vector)->data ); /* free allocated data */
  LALFree( *vector );	/* free vector struct itself */

  *vector = NULL;		/* make sure we don't point to freed struct */

  RETURN( status );
}

